/*
 * Copyright (C) 2010-2015 Max Kellermann <max.kellermann@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef STRING_API_HXX
#define STRING_API_HXX

#include "Compiler.h"

#include <string.h>

#ifdef _UNICODE
#include "WStringAPI.hxx"
#endif

gcc_pure gcc_nonnull_all
static inline size_t
StringLength(const char *p)
{
	return strlen(p);
}

gcc_pure gcc_nonnull_all
static inline const char *
StringFind(const char *haystack, const char *needle)
{
	return strstr(haystack, needle);
}

gcc_pure gcc_nonnull_all
static inline char *
StringFind(char *haystack, char needle, size_t size)
{
	return (char *)memchr(haystack, needle, size);
}

gcc_pure gcc_nonnull_all
static inline const char *
StringFind(const char *haystack, char needle, size_t size)
{
	return (const char *)memchr(haystack, needle, size);
}

gcc_pure gcc_nonnull_all
static inline const char *
StringFind(const char *haystack, char needle)
{
	return strchr(haystack, needle);
}

gcc_pure gcc_nonnull_all
static inline char *
StringFind(char *haystack, char needle)
{
	return strchr(haystack, needle);
}

gcc_pure gcc_nonnull_all
static inline const char *
StringFindLast(const char *haystack, char needle)
{
	return strrchr(haystack, needle);
}

gcc_pure gcc_nonnull_all
static inline char *
StringFindLast(char *haystack, char needle)
{
	return strrchr(haystack, needle);
}

gcc_nonnull_all
static inline void
UnsafeCopyString(char *dest, const char *src)
{
	strcpy(dest, src);
}

gcc_nonnull_all
static inline char *
UnsafeCopyStringP(char *dest, const char *src)
{
#if defined(WIN32) || defined(__BIONIC__)
  /* emulate stpcpy() */
  UnsafeCopyString(dest, src);
  return dest + StringLength(dest);
#else
  return stpcpy(dest, src);
#endif
}

/**
 * Checks whether #a and #b are equal.
 */
gcc_pure gcc_nonnull_all
static inline bool
StringIsEqual(const char *a, const char *b)
{
	return strcmp(a, b) == 0;
}

/**
 * Checks whether #a and #b are equal.
 */
gcc_pure gcc_nonnull_all
static inline bool
StringIsEqual(const char *a, const char *b, size_t length)
{
	return strncmp(a, b, length) == 0;
}

/**
 * Copy the string to a new allocation.  The return value must be
 * freed with free().
 */
gcc_malloc gcc_nonnull_all
static inline char *
DuplicateString(const char *p)
{
	return strdup(p);
}

#endif
